/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart;

import static org.fortiss.tooling.base.ui.utils.LayoutDataUtils.getNodeBounds;
import static org.fortiss.tooling.base.ui.utils.SnapToGridUtils.snapToGridAdapter;

import java.util.List;

import org.conqat.ide.commons.ui.swt.FontUtils;
import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.SnapToHelper;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.fortiss.tooling.base.layout.DefaultLayoutConstants;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.base.ui.editpart.figure.TransparentLabel;
import org.fortiss.tooling.base.ui.editpart.policy.DirectConnectionAndReconnectionEditPolicy;
import org.fortiss.tooling.base.ui.editpart.policy.ElementLayoutEditPolicy;
import org.fortiss.tooling.base.ui.editpart.policy.RemoveEditPolicy;
import org.fortiss.tooling.kernel.model.INamedCommentedElement;
import org.fortiss.tooling.kernel.ui.extension.IModelElementHandler;
import org.fortiss.tooling.kernel.ui.service.IModelEditorBindingService;
import org.fortiss.tooling.kernel.ui.service.IModelElementHandlerService;

/**
 * The base class for graphical edit parts which encapsulate nodes (i.e. the
 * main modeling elements). All of them have a name label and an activation
 * mode.
 * 
 * @param <T>
 *            the type of the model element wrapped.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 450841556180E967B0BF5BC0756859E7
 */
public abstract class ElementEditPartBase<T extends ILayoutedModelElement & INamedCommentedElement> extends
		PositionedEditPartBase<T> {

	/** The default font for titles. */
	public static Font DEFAULT_TITLE_FONT = FontUtils.derive(JFaceResources.getDefaultFont(), 0, SWT.BOLD);

	/** Constructor. */
	protected ElementEditPartBase(T modelObject) {
		super(modelObject);
	}

	/** Creates the label figure. */
	@Override
	public Label createLabelFigure() {
		Label nameLabel = new TransparentLabel();
		nameLabel.setForegroundColor(getForegroundColor());
		nameLabel.setFont(DEFAULT_TITLE_FONT);
		nameLabel.setTextAlignment(PositionConstants.LEFT | PositionConstants.TOP);
		nameLabel.setTextPlacement(PositionConstants.EAST);
		nameLabel.setIcon(IModelElementHandlerService.INSTANCE.getModelElementHandler(getModel()).getIcon());
		return nameLabel;
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * Returns the connectors as children.
	 */
	@Override
	protected List<?> getModelChildren() {
		return modelElementHandler.getConnectors(getModel());
	}

	/** {@inheritDoc} */
	@Override
	protected Rectangle determineBaseFigureBounds() {
		return getNodeBounds(getModel());
	}

	/** {@inheritDoc} */
	@Override
	protected Rectangle determineLabelFigureBounds() {
		Rectangle textBounds = labelFigure.getTextBounds();
		Rectangle labelBounds = determineBaseFigureBounds();
		labelBounds.y += (labelBounds.height - textBounds.height) / 2;
		labelBounds.x += 2 * DefaultLayoutConstants.DEFAULT_SHAPE_INSETS;
		labelBounds.width -= 4 * DefaultLayoutConstants.DEFAULT_SHAPE_INSETS;
		labelBounds.height = (int)(textBounds.height * 1.5);
		return labelBounds;
	}

	/** {@inheritDoc} */
	@Override
	protected String determineLabelFigureText() {
		return getModel().getName();
	}

	/** {@inheritDoc} */
	@Override
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new RemoveEditPolicy());
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new ElementLayoutEditPolicy<T>(this));
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE, new DirectConnectionAndReconnectionEditPolicy());
	}

	/** {@inheritDoc} */
	@Override
	public void performRequest(Request req) {
		if(req.getType() == RequestConstants.REQ_OPEN) {
			EObject eo = getModel();
			IModelElementHandler<EObject> handler = IModelElementHandlerService.INSTANCE.getModelElementHandler(eo);
			if(handler != null) {
				eo = handler.handleOpenModelElementRequest(eo);
			}
			IModelEditorBindingService.INSTANCE.openInEditor(eo);
		}
		super.performRequest(req);
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	protected void refreshVisuals() {
		super.refreshVisuals();

		for(EditPart child : (List<EditPart>)getChildren()) {
			child.refresh();
		}
	}

	/** {@inheritDoc} */
	@Override
	protected ConnectionAnchor defaultConnectionAnchor() {
		return new ChopboxAnchor(getFigure());
	}

	/** {@inheritDoc} */
	@SuppressWarnings({"rawtypes"})
	@Override
	public Object getAdapter(Class adapter) {
		if(adapter == SnapToHelper.class && getLayoutConfiguration().useGrid()) {
			return snapToGridAdapter(this);
		}
		return super.getAdapter(adapter);
	}
}
